/**
 * @module node-scribd-client
 * @requires fs
 * @requires form-data
 * @requires http
 * https://github.com/jsturgis/node-scribd-client
 *
 * Copyright (c) 2013 Jeff Sturgis
 * Licensed under the MIT license.
 */

'use strict';

var crypto = require('crypto'),
    fs = require('fs'),
    FormData = require('form-data'),
    http = require('http');

function generateSig (paramsObj, secret) {

    var str = '',
        keyArray = [],
        md5 = crypto.createHash('md5'),
        sortedParams = '';

    // extract keys into an array
    for (var key in paramsObj) {

        if( paramsObj.hasOwnProperty( key ) ) {

            keyArray.push(key);
        }
    }

    // sort keys
    keyArray.sort();

    // assemble hash string
    keyArray.forEach(function(key){
        sortedParams += key + paramsObj[key];
    });

    str = secret + sortedParams;
    md5.update(str, 'utf8');
    return md5.digest('hex');
}

function postRequest (method, params, options, secret, callback){

    var val = null,
        form = new FormData(),
        sigParams = {},
        req;

    for (var key in params) {

        if( params.hasOwnProperty( key ) ) {

            val = params[key];

            if(val){

                if (val instanceof Array) {
                    val = val.join();
                }

                if(key === 'file'){
                    form.append('file', fs.createReadStream(val));
                }else{
                    form.append(key, val);
                    sigParams[key] = val;
                }
            }
        }
    }

    form.append('api_sig', generateSig(sigParams, secret));

    form.getLength(function(err, length){

        if(err){
          return callback(err, null);
        }

        options.headers = form.getHeaders({'Content-Length': length});

        // remote http request
        req = http.request(options, function(res) {

            var responseBody = '';

            res.setEncoding('utf8');

            res.on('data', function (chunk) {
                responseBody += chunk;
            });

            res.on('end', function() {
                var data;

                try{
                    data = JSON.parse(responseBody);

                    // handle scribd errors
                    if(data && data.rsp.stat === 'fail'){
                        return callback(new Error(data.rsp.error.message), null);
                    }

                    return callback(null, data.rsp);
                }catch(err){
                    data = responseBody;
                    return callback(new Error(data), null);
                }
            });

        });

        req.on('error', function(e) {
            return callback(new Error(e.message), null);
        });

        // post the data
        form.pipe(req);
    });
}

/**
 * @constructor
 * @param {string} api_key your api key from scribd
 * @param {string} secret your secret from scribd
 * @returns {object} scribd client instance
 */
function Scribd (api_key, secret) {

    this.api_key = api_key;
    this.secret = secret;
    this.session_key = null;
    this.my_user_id = null;
    this.reqOptions = {
          hostname: 'api.scribd.com',
          port: 80,
          path: '/api',
          method: 'POST'
        };

}

// expose as a static method to generate a signature
Scribd.generateSig = generateSig;

Scribd.prototype = {
    /**
    * This method accepts the document file via Multi-Part POST.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.file relative path to file
    * @property {string} [params.doc_type] PDF, DOC, TXT, PPT, etc.
    * @property {string} [params.access] public or private. Default is Public.
    * @property {number} [params.rev_id] id of file to modify
    * @returns {object} an object containing doc_id, access_key, and secret_password if nessesary.
    */
    upload: function (params, callback) {
        var method = "docs.upload",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method retrieves the document file from the specified URL.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.url absolute URL of file
    * @property {string} params.doc_type : PDF, DOC, TXT, PPT, etc.
    * @property {string} params.access public or private. Default is Public.
    * @property {number} params.rev_id id of file to modify
    * @returns {object} an object containing doc_id, access_key, and secret_password if nessesary.
    */
    uploadFromUrl: function (params, callback) {
        var method = "docs.uploadFromUrl",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method retrieves a list of documents for a given user.
    * @param {function} callback the function you want called in response to this API call
    * @return {object} containing doc_id, title, description, access_key, and conversion_status for all documents for a given user
    */
    getList: function (callback){
        var method = "docs.getList";

        this.makeRequest(method, null, function(err, res){

            if(err){
                return callback(err, null);
            }

            callback(null, res.resultset);
        });
    },
    /**
    * This method retrieves the conversion status of the document.
    * @param {object|string} params param wrapper object or the doc_id string
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.doc_id the document id
    * @return {object} object containing "DISPLAYABLE", "DONE", "ERROR", or "PROCESSING" for the current document.
    */
    getConversionStatus: function (params, callback){
        var method = "docs.getConversionStatus",
            reqParams = params || {},
            docId;

        // single member params can be passed without wrapping in an object
        if(typeof reqParams === 'string' || typeof reqParams === 'number'){
            docId = reqParams;
            reqParams = {};
            reqParams.doc_id = docId;
        }

        this.makeRequest(method, reqParams, function(err, res){

            if(err){
                return callback(err, null);
            }

            callback(null, res);
        });
    },
    /**
    * This method retrieves the meta-data for existing documents.
    * @param {object|string} params param wrapper object or the doc_id string
    * @param {function} callback the function you want called in response to this API call
    * @property {number} params.doc_id the document id
    * @return {object} an object containing doc_id, title , description , access, tags, show_ads, license, access_key, secret_password
    */
    getSettings: function (params, callback){
        var method = "docs.getSettings",
            reqParams = params || {},
            docId;

        // single member params can be passed without wrapping in an object
        if(typeof reqParams === 'string' || typeof reqParams === 'number'){
            docId = reqParams;
            reqParams = {};
            reqParams.doc_id = docId;
        }

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method updates the meta-data for existing documents.
    * Only send arguments for fields you would like to overwrite.
    * Will return a blank response object if succesful
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {Array.<string>} params.doc_ids array of document id strings
    * @property {string} [params.title] title of document
    * @property {string} [params.description] description of document
    * @property {string} [params.access] private, or public
    * @property {string} [params.license] "by", "by-nc", "by-nc-nd", "by-nc-sa", "by-nd", "by-sa", "c" or "pd"
    * @property {array} [params.show_ads] default, true, or false
    * @property {array} [params.tags] list of tags
    * @returns {object} an empty object.
    */
    changeSettings: function (params, callback){
        var method = "docs.changeSettings",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method deletes an existing document. Will return a blank OK response object if successful.
    * @param {object|string} params param wrapper object or the doc_id string
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.doc_id the document id
    * @return {object} an empty object
    */
    delete: function (params, callback){
        var method = "docs.delete",
            reqParams = params || {},
            docId;

        // single member params can be passed without wrapping in an object
        if(typeof reqParams === 'string' || typeof reqParams === 'number'){
            docId = reqParams;
            reqParams = {};
            reqParams.doc_id = docId;
        }

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method searches for the specified query in the public documents on Scribd.com. Private documents are not searchable.
    * @param {object|string} params param wrapper object or the doc_id string
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.query search query
    * @property {number} [params.num_results] number of results to return (10 default, 1000 max)
    * @property {number} [params.num_start] number to start from
    * @property {number} [params.category_id] Restricts search results to only documents in a certain category.
    * @property {string} [params.language] Restricts search results to only documents in the specified language (in ISO 639-1 format).
    * @property {boolean} [pararms.simple] This option specifies whether or not to allow advanced search queries
    * @returns {object} results objects, each of which contain doc_id, secret password, access_key, title, and description and other meta data
    */
    search: function (params, callback){
        var method = "docs.search",
            reqParams = params || {};

        this.makeRequest(method, reqParams, function(err, res){
            if(err){
                return callback(err, null);
            }

            callback(null, res.result_set);
        });

    },
    /**
    * This method returns a link you can use to download a static version of a document.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string | number} params.doc_id Document ID of the file to download. You must have ownership of this document.
    * @property {string} params.doc_type The type of file to download. If "original", will get a link to the original file uploaded, regardless of its extension.
    * @returns {object} an object containing a link to the file
    */
    getDownloadUrl: function (params, callback){
        var method = "docs.getDownloadUrl",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method retrieves statistics of the document.
    * @param {object|string} params param wrapper object or the doc_id string
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.doc_id the document id
    * @returns {object} an object containing a single property "reads" with the number numb of reads.
    */
    getStats: function (params, callback){
        var method = "docs.getStats",
            reqParams = params || {},
            docId;

        // single member params can be passed without wrapping in an object
        if(typeof reqParams === 'string' || typeof reqParams === 'number'){
            docId = reqParams;
            reqParams = {};
            reqParams.doc_id = docId;
        }

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method returns a list of categories or subcategories.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} [params.category_id] The ID of category to retrieve children for. If none - all root categories will be returned.
    * @property {boolean|string} [params.with_subcategories] Include subcategories in results.
    * @returns {object} an object containing the result_set object.
    */
    getCategories: function (params, callback){
        var method = "docs.getCategories",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method returns a list of featured documents.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} [params.limit] Number of results to return: Default 20, max 1000.
    * @property {boolean|string} [params.offset] Number to start at: Default 0. Cannot exceed 1000.
    * @property {string} [params.scope] Whether to search new documents or hot only. Set to "hot" by default.
    * @returns {object} an object containing the result_set object.
    */
    featured: function (params, callback){
        var method = "docs.featured",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method returns a list of documents that meet filter criteria.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} [params.limit] Number of results to return: Default 20, max 1000.
    * @property {boolean|string} [params.offset] Number to start at: Default 1. Cannot exceed 1000.
    * @property {number|string} [params.category_id] A category ID to search documents in.
    * @property {string} [params.sort] Sort order. Set to "popular" by default.
    * @returns {object} an object containing the result_set object.
    */
    browse: function (params, callback){
        var method = "docs.browse",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method accepts a document thumbnail file via Multi-Part POST.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.file relative path to file
    * @property {number|string} params.doc_id The doc_id to attach the uploaded thumbnail to.
    * @returns {object} an object containing the stat property.
    */
    uploadThumb: function (params, callback){
        var method = "docs.uploadThumb",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method retrieves a URL to the thumbnail of a document, in a given size.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.doc_id The ID of the document. It must be a public document, or one that your account controls.
    * @property {number|string} [params.width] Width in px of the desired image. If not included, will use the default thumb size.
    * @returns {object} an object containing the thumbnail_url property.
    */
    getThumb: function (params, callback){
        var method = "thumbnail.get",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method allows your API application to sign in as an existing Scribd user, executing methods as that user.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.username username of user to log in
    * @property {string} params.password password of user to log in
    * @returns {object} an object containing session_key, name, username, and user_id of the user
    */
    login: function (params, callback){
        var method = "user.login",
            reqParams = params || {},
            that = this;

        this.makeRequest(method, reqParams, function(err, res){
            if(err){
                return callback(err, null);
            }

            that.session_key = res.session_key;

            callback(null, res);
        });
    },
    /**
    * This method allows your API application to signup a new Scribd user. If the signup is successful,
    * your application will be passed back a session key which will allow you to execute methods on behalf of the new user.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.username username of user to log in
    * @property {string} params.password password of user to log in
    * @property {string} params.email email address of user
    * @property {string} params.name name of user
    * @returns {object} an object containing session_key, name, username, and user_id of the user
    */
    signup: function(params, callback){
        var method = "user.signup",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method returns a URL that, when visited, will automatically sign in the given user
    * account and then redirect to the URL you provide. The URL expires after 7 days
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.next_url The URL or path portion of a Scribd URL to redirect to. Set to blank for home page.
    * @returns {object} an object containing the auto-signin URL prop named url
    */
    getAutoSigninUrl: function(params, callback){
        var method = "user.getAutoSigninUrl",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method adds a document to an existing collection. Will return a blank OK response object if successful.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.doc_id The Doc id
    * @property {number|string} params.collection_id The id of the collection. This can be obtained with a call to getCollections.
    * @returns {object} an object containing the stat property
    */
    addDocToCollection: function(params, callback){
        var method = "collections.addDoc",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method creates a new collection. Will return an OK response collection_id and collection_type.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} params.name The name of the collection.
    * @property {string} [params.description] The description of the collection.
    * @property {string} [params.privacy_type] This specifies the privacy type of the collection. This value can either be "public" or "private". If not included, the collection will default to "public."
    * @returns {object} an object containing the collection_id and privacy_type
    */
    createCollection: function(params, callback){
        var method = "collections.create",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method updates a new collection's name, description or privacy_type. Will return an OK response collection_id and changed attributes and their corresponding values.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.collection_id The id of the collection.
    * @property {string} [params.name] The name of the collection. When present, it cannot be blank or a string containing only whitespaces.
    * @property {string} [params.description] The description of the collection.
    * @returns {object} an object containing the collection_id and changed attributes
    */
    updateCollection: function(params, callback){
        var method = "collections.update",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method deletes new collection. Will return a blank OK response object if successful.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.collection_id The id of the collection.
    * @returns {object} an object containing the stat property
    */
    deleteCollection: function(params, callback){
        var method = "collections.delete",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method retrieves a list of collections for a given user.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {string} [params.scope] This specifies the privacy type of the collections that are returned. This value can either be "public" or "private. If not included, all document collections are returned.
    * @returns {object} an object containing the result_set
    */
    getCollectionList: function(params, callback){
        var method = "collections.getList",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method removes a document from an existing collection. Will return a blank OK response object if successful.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.doc_id the Doc id to remove
    * @property {number|string} params.collection_id The id of the collection.
    * @returns {object} an object containing the stat prop
    */
    removeDocFromCollection: function(params, callback){
        var method = "collections.removeDoc",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    /**
    * This method retrieves a list of documents in a given collection.
    * @param {object} params param wrapper object
    * @param {function} callback the function you want called in response to this API call
    * @property {number|string} params.collection_id The id of the collection.
    * @property {number|string} [params.limit] The id of the collection.
    * @property {number|string} [params.offset]  The offset into the list of documents. You can paginate through the full list using the limit and offset parameters.
    * @returns {object} an object containing the result_set
    */
    listCollectionDocs: function(params, callback){
        var method = "collections.listDocs",
            reqParams = params || {};

        this.makeRequest(method, reqParams, callback);
    },
    makeRequest: function(method, params, callback){
        var secret,
            options = this.reqOptions,
            reqParams = params || {};

        reqParams.method = method;
        reqParams.session_key = this.session_key;
        reqParams.my_user_id = this.my_user_id;
        reqParams.format = 'json';
        reqParams.api_key = this.api_key;

        secret = this.secret;

        postRequest(method, reqParams, options, secret, callback);
    }
};

module.exports = Scribd;