'use strict';

var Scribd = require('../lib/node-scribd-client.js'),
    fs = require('fs'),
    nock = require('nock');

function getMockResponse(method) {
    var filename = __dirname + '/mocks/' + method + '.json',
        data = fs.readFileSync(filename, {encoding: 'utf8'});

    return JSON.parse(data);
}

function testResonponseData(test, mockResponse, actualResponse) {
    for(var field in mockResponse){
        test.deepEqual(actualResponse[field], mockResponse[field], 'response should have field ' + field + ' and the values should match');
    }
}

/*
  ======== A Handy Little Nodeunit Reference ========
  https://github.com/caolan/nodeunit

  Test methods:
    test.expect(numAssertions)
    test.done()
  Test assertions:
    test.ok(value, [message])
    test.equal(actual, expected, [message])
    test.notEqual(actual, expected, [message])
    test.deepEqual(actual, expected, [message])
    test.notDeepEqual(actual, expected, [message])
    test.strictEqual(actual, expected, [message])
    test.notStrictEqual(actual, expected, [message])
    test.throws(block, [error], [message])
    test.doesNotThrow(block, [error], [message])
    test.ifError(value)
*/

exports['scribd'] = {
    setUp: function(done) {
        // setup here
        var scribdApiKey = "YOUR-API-KEY",
            scribdSecret = "YOUR-SECRET";

        this.scribd = new Scribd(scribdApiKey, scribdSecret);
        this.docId = '159857500';

        this.scope = nock('http://api.scribd.com')
                        .post('/api');

        done();
    },
    'get list': function(test) {

        var method = 'docs.getList',
            mockResponse = getMockResponse(method);

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getList(function(err, res){
            test.ifError(err);
            test.equal(res.result.length, mockResponse.resultset.result.length, 'should have two results.');
            test.equal(res.result[0].doc_id, mockResponse.resultset.result[0].doc_id, 'should have a doc_id.');
            test.done();
        });
    },
    'upload file': function(test) {

        var file = __filename,
            doc_type = 'txt',
            access = null,
            rev_id = null,
            method = 'docs.upload',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {
                file: file,
                doc_type: doc_type,
                access: access,
                rev_id: rev_id
            };

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.upload(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'upload from url': function (test) {

        this.scribd.my_user_id = '143';
        var url = 'http://lib.store.yahoo.net/lib/paulgraham/onlisp.ps',
            doc_type = 'txt',
            access = 'public',
            rev_id = null,
            method = 'docs.uploadFromUrl',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {
                url: url,
                doc_type: doc_type,
                access: access,
                rev_id: rev_id
            };

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.uploadFromUrl(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },

    'get conversion status': function (test){

        var method = 'docs.getConversionStatus',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: this.docId};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getConversionStatus(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'get settings': function (test){
        var method = 'docs.getSettings',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: this.docId};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getSettings(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'change settings': function (test){
        var method = 'docs.changeSettings',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_ids: [this.docId]};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.changeSettings(params, function(err, res){
            test.ifError(err);
            test.equal(res.stat, 'ok', 'stat should be ok');
            test.done();
        });
    },
    delete: function (test){
        var method = 'docs.delete',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: this.docId};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.delete(params, function(err, res){
            test.ifError(err);
            test.equal(res.stat, 'ok', 'stat should be ok');
            test.done();
        });
    },
    search: function (test){
        var method = 'docs.search',
            mockResponse = getMockResponse(method),
            params = {doc_id: this.docId};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.search(params, function(err, res){
            test.ifError(err);
            test.equal(res.result.length, mockResponse.result_set.result.length, 'should have two results.');
            test.equal(res.result[0].title, mockResponse.result_set.result[0].title, 'should have a title.');
            test.done();
        });

    },
    'get download url': function (test){
        var method = 'docs.getDownloadUrl',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: this.docId, doc_type: 'txt'};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getDownloadUrl(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'get stats': function (test){
        var method = 'docs.getStats',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: mockResponse.doc_id};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getStats(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'get categories': function (test){
        var method = "docs.getCategories",
            mockResponse = getMockResponse(method),
            params = {};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getCategories(params, function(err, res){
            test.ifError(err);
            test.equal(res.result_set.result.length, mockResponse.result_set.result.length, 'should have '+mockResponse.result_set.result.length+' results.');
            test.equal(res.result_set.result[0].id, mockResponse.result_set.result[0].id, 'should have a id.');
            test.done();
        });
    },
    'featured': function (test){
        var method = "docs.featured",
            mockResponse = getMockResponse(method),
            params = {};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getCategories(params, function(err, res){
            test.ifError(err);
            test.equal(res.result_set.result.length, mockResponse.result_set.result.length, 'should have '+mockResponse.result_set.result.length+' results.');
            test.equal(res.result_set.result[0].title, mockResponse.result_set.result[0].title, 'should have a title.');
            test.done();
        });
    },
    'browse': function (test){
        var method = "docs.browse",
            mockResponse = getMockResponse(method),
            params = {};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getCategories(params, function(err, res){
            test.ifError(err);
            test.equal(res.result_set.result.length, mockResponse.result_set.result.length, 'should have '+mockResponse.result_set.result.length+' results.');
            test.equal(res.result_set.result[0].title, mockResponse.result_set.result[0].title, 'should have a title.');
            test.done();
        });
    },
    'upload thumb': function (test){
        var method = 'docs.uploadThumb',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: this.docId, file: __filename};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.uploadThumb(params, function(err, res){
            test.ifError(err);
            test.equal(res.stat, 'ok', 'stat should be ok');
            test.done();
        });
    },
    'get thumb': function (test){
        var method = 'thumbnail.get',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {doc_id: this.docId, width: 20};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getThumb(params, function(err, res){
            test.ifError(err);
            test.ok(res.stat, 'stat should exist');
            test.equal(res.thumbnail_url, mockResponse.thumbnail_url, 'stat have a thumbnail_url prop');
            test.done();
        });
    },
    login: function (test){
        var method = 'user.login',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {username: 'foo', password: 'bar'};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.login(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    signup: function(test){
        var method = 'user.signup',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {
                    username: 'foo',
                    password: 'bar',
                    name: 'foo bar',
                    email: 'foobar@gmail.com'
                };

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.signup(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'get auto signin url': function(test){
        var method = 'user.getAutoSigninUrl',
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {
                    next_url: 'http://www.google.com',
                };

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getAutoSigninUrl(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'add doc to collection': function(test){
        var method = "collections.addDoc",
            mockResponse = getMockResponse('api.statOk'),
            params = {doc_id: this.doc_id, collection_id: '123'};

        test.expect(2);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.addDocToCollection(params, function(err, res){
            test.ifError(err);
            test.ok(res.stat, 'stat should be ok');
            test.done();
        });
    },
    'create collection': function(test){
        var method = "collections.create",
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {name: 'collection'};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.createCollection(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'update collection': function(test){
        var method = "collections.update",
            mockResponse = getMockResponse(method),
            expect = Object.keys(mockResponse).length + 1,
            params = {collection_id: '123'};

        test.expect(expect);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.updateCollection(params, function(err, res){
            test.ifError(err);
            testResonponseData(test, mockResponse, res);
            test.done();
        });
    },
    'delete collection': function(test){
        var method = "collections.delete",
            mockResponse = getMockResponse('api.statOk'),
            params = {collection_id: '123'};

        test.expect(2);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.deleteCollection(params, function(err, res){
            test.ifError(err);
            test.ok(res.stat, 'stat should be ok');
            test.done();
        });
    },
    'get collection list': function(test){
        var method = "collections.getList",
            mockResponse = getMockResponse(method),
            params = {};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getCollectionList(params, function(err, res){
            test.ifError(err);
            test.equal(res.resultset.result.length, mockResponse.resultset.result.length, 'should have '+mockResponse.resultset.result.length+' results.');
            test.equal(res.resultset.result[0].collection_id, mockResponse.resultset.result[0].collection_id, 'should have a collection_id.');
            test.done();
        });
    },
    'remove doc from collection': function(test){
        var method = "collections.removeDoc",
            mockResponse = getMockResponse('api.statOk'),
            params = {doc_id: this.doc_id, collection_id: '123'};

        test.expect(2);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.removeDocFromCollection(params, function(err, res){
            test.ifError(err);
            test.ok(res.stat, 'stat should be ok');
            test.done();
        });
    },
    'list collection docs': function(test){
        var method = "collections.listDocs",
            mockResponse = getMockResponse(method),
            params = {collection_id: '123'};

        test.expect(3);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.listCollectionDocs(params, function(err, res){
            test.ifError(err);
            test.equal(res.result_set.result.length, mockResponse.result_set.result.length, 'should have '+mockResponse.result_set.result.length+' results.');
            test.equal(res.result_set.result[0].title, mockResponse.result_set.result[0].title, 'should have a title.');
            test.done();
        });
    },
    'api error': function(test){
        var mockResponse = getMockResponse('api.error'),
            params = {};

        test.expect(1);

        this.scope.reply(200, {rsp: mockResponse});

        this.scribd.getConversionStatus(params, function(err, res){
            test.ok(err, 'should have an error response');
            test.done();
        });
    },
    'api signature test': function(test){
        var knownSig = '47f185afd858eaa4d33788d84c947ada',
            params = {c: 'test', a: 'test'},
            secret = 'abc',
            generatedSig = Scribd.generateSig(params, secret);

        test.expect(1);
        test.equal(generatedSig, knownSig, 'known signature should match generated signature');
        test.done();
    }
};
